import { useCallback, useContext } from 'react';
import { PropTypes } from 'prop-types';
import Button from 'react-bootstrap/Button';

import Web3 from 'web3';
import Web3Context from './Web3Context';

const getRequestStatus = (request) => {
  switch (Number(request.status)) {
    case 0:
      return 'Requested';
    case 1:
      return 'Evidence Submitted';
    case 2:
      return 'Approval Submitted';
    case 3:
      return 'Complete';
    default:
      return 'Unknown';
  }
};

function WorkRequests({ showRequestWork }) {
  const {
    provider, work1, work1Price, workRequests, availabilityStakes, account,
  } = useContext(Web3Context);

  const requestWork = useCallback(async () => {
    const web3 = new Web3(provider);
    const priceWei = BigInt(web3.utils.toWei(work1Price, 'ether'));
    await work1.methods.requestWork().send({
      from: account,
      gas: 1000000,
      value: priceWei,
    });
  }, [provider, work1, account, work1Price]);

  const submitWorkEvidence = useCallback(async (requestIndex) => {
    await work1.methods.submitWorkEvidence(requestIndex).send({
      from: account,
      gas: 1000000,
    });
  }, [work1, account]);

  const submitWorkApproval = useCallback(async (requestIndex) => {
    await work1.methods.submitWorkApproval(requestIndex, true).send({
      from: account,
      gas: 1000000,
    });
  }, [work1, account]);

  const submitWorkDisapproval = useCallback(async (requestIndex) => {
    await work1.methods.submitWorkApproval(requestIndex, false).send({
      from: account,
      gas: 1000000,
    });
  }, [work1, account]);

  return (
    <>
      {showRequestWork && (
        <div>
          <Button onClick={() => requestWork()}>Request Work</Button>
        </div>
      )}
      <div>
        Work Request Count:
        {' '}
        {workRequests.length}
      </div>
      <div>
        <table className="table">
          <thead>
            <tr>
              <th>ID</th>
              <th>Customer</th>
              <th>Fee</th>
              <th>Status</th>
              <th>Stake ID</th>
              <th>Approval</th>
              <th>Pool ID</th>
              <th>Actions</th>
            </tr>
          </thead>
          <tbody>
            {workRequests.filter((x) => !!x).map((request) => (
              <tr key={request.id}>
                <td>{request.id.toString()}</td>
                <td>{request.customer.toString()}</td>
                <td>
                  {request.feeEther}
                  {' '}
                  ETH
                </td>
                <td>{getRequestStatus(request)}</td>
                <td>{request.stakeIndex.toString()}</td>
                <td>{request.approval.toString()}</td>
                <td>{request.poolIndex.toString()}</td>
                <td>
                  {availabilityStakes.length > 0
                          && availabilityStakes[Number(request.stakeIndex)]?.currentUserIsWorker()
                          && Number(request.status) === 0 && (
                          <Button onClick={() => submitWorkEvidence(request.id)}>
                            Submit Work Evidence
                          </Button>
                  )}
                  {request.currentUserIsCustomer()
                          && Number(request.status) === 1 && (
                            <>
                              <Button onClick={() => submitWorkApproval(request.id)}>
                                Submit Approval
                              </Button>
                              <Button onClick={() => submitWorkDisapproval(request.id)}>
                                Submit Dispproval
                              </Button>
                            </>
                  )}
                </td>
              </tr>
            ))}
          </tbody>
        </table>
      </div>
    </>
  );
}

WorkRequests.propTypes = {
  showRequestWork: PropTypes.bool,
};

WorkRequests.defaultProps = {
  showRequestWork: false,
};

export default WorkRequests;
